@@ -3,7 +3,7 @@
  *		that either displays or sets the characteristics of
  *		one or more of the system's networking interfaces.
  *
- * Version:	ifconfig 1.27 (1997-08-03)
+ * Version:	ifconfig 1.33 (1998-03-02)
  *
  * Author:	Fred N. van Kempen, <waltje@uwalt.nl.mugnet.org>
  *              and others.  Copyright 1993 MicroWalt Corporation
@@ -38,6 +38,7 @@
 
 #if HAVE_AFINET6
 
+#ifndef _LINUX_IN6_H
 /*
  *	This is in linux/include/net/ipv6.h.
  */
@@ -47,6 +48,8 @@ struct in6_ifreq {
   __u32 ifr6_prefixlen;
   unsigned int ifr6_ifindex;
 };
+
+#endif
  
 #define IPV6_ADDR_ANY		0x0000U
 
@@ -67,68 +70,12 @@ struct in6_ifreq {
 
 #endif  /* HAVE_AFINET6 */
 
-#ifndef IFF_PORTSEL
-#define IFF_PORTSEL	0x2000
-#define IFF_AUTOMEDIA	0x4000
+/* Check for supported features */
+
+#if defined(SIOCSIFTXQLEN) && defined(ifr_qlen)
+#define HAVE_TXQUEUELEN
 #endif
 
-/* This is from <linux/netdevice.h>. */
-
-struct user_net_device_stats
-{
-  unsigned long	rx_packets;	/* total packets received	*/
-  unsigned long	tx_packets;	/* total packets transmitted	*/
-  unsigned long	rx_bytes;	/* total bytes received 	*/
-  unsigned long	tx_bytes;	/* total bytes transmitted	*/
-  unsigned long	rx_errors;	/* bad packets received		*/
-  unsigned long	tx_errors;	/* packet transmit problems	*/
-  unsigned long	rx_dropped;	/* no space in linux buffers	*/
-  unsigned long	tx_dropped;	/* no space available in linux	*/
-  unsigned long	multicast;	/* multicast packets received	*/
-  unsigned long	collisions;
-
-  /* detailed rx_errors: */
-  unsigned long	rx_length_errors;
-  unsigned long	rx_over_errors;	/* receiver ring buff overflow	*/
-  unsigned long	rx_crc_errors;	/* recved pkt with crc error	*/
-  unsigned long	rx_frame_errors; /* recv'd frame alignment error */
-  unsigned long	rx_fifo_errors;	/* recv'r fifo overrun		*/
-  unsigned long	rx_missed_errors; /* receiver missed packet	*/
-  /* detailed tx_errors */
-  unsigned long	tx_aborted_errors;
-  unsigned long	tx_carrier_errors;
-  unsigned long	tx_fifo_errors;
-  unsigned long	tx_heartbeat_errors;
-  unsigned long	tx_window_errors;
-};
-
-struct interface {
-  char			name[IFNAMSIZ];		/* interface name	 */
-  short			type;			/* if type		 */
-  short			flags;			/* various flags	 */
-  int			metric;			/* routing metric	 */
-  int			mtu;			/* MTU value		 */
-  struct ifmap		map;			/* hardware setup	 */
-  struct sockaddr	addr;			/* IP address		 */
-  struct sockaddr	dstaddr;		/* P-P IP address	 */
-  struct sockaddr	broadaddr;		/* IP broadcast address	 */
-  struct sockaddr	netmask;		/* IP network mask	 */
-  struct sockaddr	ipxaddr_bb;		/* IPX network address   */
-  struct sockaddr	ipxaddr_sn;		/* IPX network address   */
-  struct sockaddr	ipxaddr_e3;		/* IPX network address   */
-  struct sockaddr	ipxaddr_e2;		/* IPX network address   */
-  struct sockaddr	ddpaddr;		/* Appletalk DDP address */
-  int			has_ip;
-  int			has_ipx_bb;
-  int			has_ipx_sn;
-  int			has_ipx_e3;
-  int			has_ipx_e2;
-  int			has_ax25;
-  int			has_ddp;
-  char			hwaddr[32];		/* HW address		 */
-  struct user_net_device_stats stats;		/* statistics		 */
-};
-
 static const char *if_port_text[][4] = {
   /* Keep in step with <linux/netdevice.h> */
   { "unknown", NULL , NULL, NULL },
@@ -141,17 +88,6 @@ static const char *if_port_text[][4] = {
   { NULL, NULL, NULL, NULL },
 };
 
-/* Not all kernels have these new ioctls. */
-#ifndef SIOGIFNAME
-#define SIOGIFNAME	0x8934		/* if_index -> name mapping	*/
-#endif
-#ifndef SIOCGIFCOUNT
-#define SIOCGIFCOUNT	0x8935		/* get number of interfaces	*/
-#endif
-#ifndef SIOCDIFADDR
-#define SIOCDIFADDR	0x8936		/* delete PA address		*/
-#endif
-
 #if HAVE_AFIPX
 #include "ipx.h"
 #endif
@@ -159,31 +95,16 @@ static const char *if_port_text[][4] = {
 #include "pathnames.h"
 #include "version.h"
 #include "net-locale.h"
+#include "interface.h"
+#include "sockets.h"
 
 char *Release = RELEASE,
-     *Version = "ifconfig 1.27 (1997-08-03)";
-
+     *Version = "ifconfig 1.33 (1998-03-02)";
 
 int opt_a = 0;				/* show all interfaces		*/
 int opt_i = 0;				/* show the statistics		*/
 int opt_v = 0;				/* debugging output flag	*/
 
-int skfd = -1;				/* generic raw socket desc.	*/
-#if HAVE_AFIPX
-int ipx_sock = -1;			/* IPX socket			*/
-#endif
-#if HAVE_AFAX25
-int ax25_sock = -1;			/* AX.25 socket			*/
-#endif
-#if HAVE_AFINET
-int inet_sock = -1;			/* INET socket			*/
-#endif
-#if HAVE_AFINET6
-int inet6_sock = -1;			/* INET6 socket			*/
-#endif
-#if HAVE_AFATALK
-int ddp_sock = -1;			/* Appletalk DDP socket		*/
-#endif
 int addr_family = 0;			/* currently selected AF	*/
 
 
@@ -193,7 +114,7 @@ ife_print(struct interface *ptr)
   struct aftype *ap;
   struct hwtype *hw;
   int hf;
-  char *dispname=NLS_CATSAVE (catfd, ifconfigSet, ifconfig_over, "overruns");
+  int can_compress = 0;
   static struct aftype *ipxtype=NULL, *ddptype=NULL;
 #if HAVE_AFINET6
   FILE *f;
@@ -212,48 +133,45 @@ ife_print(struct interface *ptr)
 
   hf=ptr->type;
 
-  if(strncmp(ptr->name,"lo",2)==0)
+  if (strncmp(ptr->name, "lo", 2) == 0)
   	hf=255;
   	
-  if(hf==ARPHRD_CSLIP || hf==ARPHRD_CSLIP6)
-  {
-#if NLS
-    /* NLS must free dispname */
-    free (dispname);
-#endif
-    /* Overrun got reused: BAD - fix later */
-    dispname=NLS_CATSAVE (catfd, ifconfigSet, ifconfig_compress, "compressed");
-  }
+  if (hf==ARPHRD_CSLIP || hf==ARPHRD_CSLIP6)
+    can_compress = 1;
   
   hw = get_hwntype(hf);
   if (hw == NULL) hw = get_hwntype(-1);
 
   printf(NLS_CATGETS(catfd, ifconfigSet, ifconfig_link, 
 		     "%-8.8s  Link encap:%s  "), ptr->name, hw->title);
-  /* Don't print the hardware address for ATM if it's null. */
-  if (hw->sprint != NULL && (strncmp(ptr->name, "atm", 3) || 
+  /* Don't print the hardware address for ATM or Ash if it's null. */
+  if (hw->sprint != NULL && ((strncmp(ptr->name, "atm", 3) &&
+			      strncmp(ptr->name, "ash", 3)) || 
    (ptr->hwaddr[0] || ptr->hwaddr[1] || ptr->hwaddr[2] || ptr->hwaddr[3] || 
    ptr->hwaddr[4] || ptr->hwaddr[5]))) 
     printf(NLS_CATGETS(catfd, ifconfigSet, ifconfig_hw, "HWaddr %s  ")
 	   , hw->print(ptr->hwaddr));
+#ifdef IFF_PORTSEL
   if (ptr->flags & IFF_PORTSEL) 
     printf("Media:%s%s", if_port_text[ptr->map.port][0], 
 	   (ptr->flags & IFF_AUTOMEDIA)?"(auto)":"");
+#endif
   printf("\n");
 #if HAVE_AFINET6  
   if (ap->af != AF_INET6) {
 #endif
     printf(NLS_CATGETS(catfd, ifconfigSet, ifconfig_adr,
-		       "          %s addr:%s"), ap->name,
+		       "          %s addr:%s "), ap->name,
 	   ap->sprint(&ptr->addr, 1));
     if (ptr->flags & IFF_POINTOPOINT) {
-      printf(NLS_CATGETS(catfd, ifconfigSet, ifconfig_pap, "  P-t-P:%s  "),
+      printf(NLS_CATGETS(catfd, ifconfigSet, ifconfig_pap, " P-t-P:%s "),
 	     ap->sprint(&ptr->dstaddr, 1));
-    } else {
-      printf(NLS_CATGETS(catfd, ifconfigSet, ifconfig_bcast, "  Bcast:%s  "),
+    } 
+    if (ptr->flags & IFF_BROADCAST) {
+      printf(NLS_CATGETS(catfd, ifconfigSet, ifconfig_bcast, " Bcast:%s "),
 	     ap->sprint(&ptr->broadaddr, 1));
     }
-    printf(NLS_CATGETS(catfd, ifconfigSet, ifconfig_mask, "Mask:%s\n"),
+    printf(NLS_CATGETS(catfd, ifconfigSet, ifconfig_mask, " Mask:%s\n"),
 	   ap->sprint(&ptr->netmask, 1));
 #if HAVE_AFINET6
   }
@@ -334,26 +252,33 @@ ife_print(struct interface *ptr)
   if (ptr->flags & IFF_MULTICAST) printf("MULTICAST ");
   printf(NLS_CATGETS(catfd, ifconfigSet, ifconfig_mtu, " MTU:%d  Metric:%d\n"),
 	 ptr->mtu, ptr->metric?ptr->metric:1);
-
+  if (ptr->tx_queue_len != -1)
+    printf("          txqueuelen:%d\n", ptr->tx_queue_len);
 
   /* If needed, display the interface statistics. */
   printf("          ");
 
   printf(NLS_CATGETS(catfd, ifconfigSet, ifconfig_rx,
-	"RX packets:%lu errors:%lu dropped:%lu %s:%lu frame:%lu\n"),
+	"RX packets:%lu errors:%lu dropped:%lu overruns:%lu frame:%lu\n"),
 	ptr->stats.rx_packets, ptr->stats.rx_errors,
-	ptr->stats.rx_dropped, dispname, ptr->stats.rx_fifo_errors,
+	ptr->stats.rx_dropped, ptr->stats.rx_fifo_errors,
         ptr->stats.rx_frame_errors);
+  if (can_compress)
+    printf("             compressed:%lu\n", ptr->stats.rx_compressed);
 	 
   printf("          ");
 
   printf(NLS_CATGETS(catfd, ifconfigSet, ifconfig_tx,
-	"TX packets:%lu errors:%lu dropped:%lu %s:%lu carrier:%lu coll:%lu\n"),
+	"TX packets:%lu errors:%lu dropped:%lu overruns:%lu carrier:%lu\n"),
 	ptr->stats.tx_packets, ptr->stats.tx_errors,
-	ptr->stats.tx_dropped, dispname, ptr->stats.tx_fifo_errors,
-	ptr->stats.tx_carrier_errors, ptr->stats.collisions);
+	ptr->stats.tx_dropped, ptr->stats.tx_fifo_errors,
+	ptr->stats.tx_carrier_errors);
+  printf("             collisions:%lu ", ptr->stats.collisions);
+  if (can_compress)
+    printf("compressed:%lu ", ptr->stats.tx_compressed);
+  printf("\n");
 
-  if (hf<255 && (ptr->map.irq || ptr->map.mem_start || ptr->map.dma || 
+  if ((ptr->map.irq || ptr->map.mem_start || ptr->map.dma || 
 		ptr->map.base_addr)) {
     printf("          ");
     if (ptr->map.irq)
@@ -374,246 +299,35 @@ ife_print(struct interface *ptr)
   }
   
   printf("\n");
-
-#if NLS
-  /* NLS must free dispname */
-  free (dispname);
-#endif
 }
 
-
-static void if_getstats(char *ifname, struct interface *ife)
-{
-  FILE *f = fopen("/proc/net/dev", "r");
-  char buf[256];
-  int have_byte_counters = 0;
-  char *bp;
-  if (f==NULL)
-    return;
-  fgets(buf, 255, f);  /* throw away first line of header */
-  fgets(buf, 255, f);
-  if (strstr(buf, "bytes")) have_byte_counters=1;
-  while(fgets(buf,255,f)) {
-    bp=buf;
-    while(*bp&&isspace(*bp))
-      bp++;
-    if(strncmp(bp,ifname,strlen(ifname))==0 && bp[strlen(ifname)]==':') {
-      bp=strchr(bp,':');
-      bp++;
-      if (have_byte_counters) {
-	sscanf(bp,"%ld %ld %ld %ld %ld %ld %ld %ld %ld %ld %ld %ld %ld",
-	       &ife->stats.rx_bytes,
-	       &ife->stats.rx_packets,
-	       &ife->stats.rx_errors,
-	       &ife->stats.rx_dropped,
-	       &ife->stats.rx_fifo_errors,
-	       &ife->stats.rx_frame_errors,
-	       
-	       &ife->stats.tx_bytes,
-	       &ife->stats.tx_packets,
-	       &ife->stats.tx_errors,
-	       &ife->stats.tx_dropped,
-	       &ife->stats.tx_fifo_errors,
-	       &ife->stats.collisions,
-	       
-	       &ife->stats.tx_carrier_errors
-	       );
-      } else {
-	sscanf(bp,"%ld %ld %ld %ld %ld %ld %ld %ld %ld %ld %ld",
-	       &ife->stats.rx_packets,
-	       &ife->stats.rx_errors,
-	       &ife->stats.rx_dropped,
-	       &ife->stats.rx_fifo_errors,
-	       &ife->stats.rx_frame_errors,
-	       
-	       &ife->stats.tx_packets,
-	       &ife->stats.tx_errors,
-	       &ife->stats.tx_dropped,
-	       &ife->stats.tx_fifo_errors,
-	       &ife->stats.collisions,
-	       
-	       &ife->stats.tx_carrier_errors
-	       );
-	ife->stats.rx_bytes = 0;
-	ife->stats.tx_bytes = 0;
-      }
-      break;
-    }
-  }
-  fclose(f);
-}
-
-/* Support for fetching an IPX address */
-
-#if HAVE_AFIPX
-static int ipx_getaddr(int sock, int ft, struct ifreq *ifr)
-{
-	((struct sockaddr_ipx *)&ifr->ifr_addr)->sipx_type=ft;
-	return ioctl(sock, SIOCGIFADDR, ifr);
-}
-#endif
-
-/* Fetch the interface configuration from the kernel. */
-static int
-if_fetch(char *ifname, struct interface *ife)
-{
-  struct ifreq ifr;
-
-  memset((char *) ife, 0, sizeof(struct interface));
-  strcpy(ife->name, ifname);
-
-  strcpy(ifr.ifr_name, ifname);
-  if (ioctl(skfd, SIOCGIFFLAGS, &ifr) < 0) return(-1);
-  ife->flags = ifr.ifr_flags;
-
-  strcpy(ifr.ifr_name, ifname);
-  if (ioctl(skfd, SIOCGIFHWADDR, &ifr) < 0) 
-    memset(ife->hwaddr, 0, 32);
-  else 
-    memcpy(ife->hwaddr,ifr.ifr_hwaddr.sa_data,8);
-
-  ife->type=ifr.ifr_hwaddr.sa_family;
-
-  strcpy(ifr.ifr_name, ifname);
-  if (ioctl(skfd, SIOCGIFMETRIC, &ifr) < 0)
-    ife->metric = 0;
-  else 
-    ife->metric = ifr.ifr_metric;
-
-  strcpy(ifr.ifr_name, ifname);
-  if (ioctl(skfd, SIOCGIFMTU, &ifr) < 0)
-    ife->mtu = 0;
-  else 
-    ife->mtu = ifr.ifr_mtu;
-
-  strcpy(ifr.ifr_name, ifname);
-  if (ioctl(skfd, SIOCGIFMAP, &ifr) < 0)
-    memset(&ife->map, 0, sizeof(struct ifmap));
-  else 
-    memcpy(&ife->map,&ifr.ifr_map,sizeof(struct ifmap));
-
-  strcpy(ifr.ifr_name, ifname);
-  if (ioctl(skfd, SIOCGIFMAP, &ifr) < 0)
-    memset(&ife->map, 0, sizeof(struct ifmap));
-  else 
-    ife->map = ifr.ifr_map;
-  
-#if HAVE_AFINET
-  strcpy(ifr.ifr_name, ifname);
-  if (inet_sock < 0 || ioctl(inet_sock, SIOCGIFDSTADDR, &ifr) < 0)
-    memset(&ife->dstaddr, 0, sizeof(struct sockaddr));
-  else 
-    ife->dstaddr = ifr.ifr_dstaddr;
-
-  strcpy(ifr.ifr_name, ifname);
-  if (inet_sock < 0 || ioctl(inet_sock, SIOCGIFBRDADDR, &ifr) < 0)
-    memset(&ife->broadaddr, 0, sizeof(struct sockaddr));
-  else 
-    ife->broadaddr = ifr.ifr_broadaddr;
-
-  strcpy(ifr.ifr_name, ifname);
-  if (inet_sock < 0 || ioctl(inet_sock, SIOCGIFNETMASK, &ifr) < 0)
-    memset(&ife->netmask, 0, sizeof(struct sockaddr));
-  else 
-    ife->netmask = ifr.ifr_netmask;
-
-  strcpy(ifr.ifr_name, ifname);
-  if (inet_sock < 0 || ioctl(inet_sock, SIOCGIFADDR, &ifr) < 0) 
-    memset(&ife->addr, 0, sizeof(struct sockaddr));
-  else 
-    ife->addr = ifr.ifr_addr;
-#endif
-  
-#if HAVE_AFATALK
-  /* DDP address maybe ? */
-  strcpy(ifr.ifr_name, ifname);
-  if (ddp_sock >= 0 && ioctl(ddp_sock, SIOCGIFADDR, &ifr) == 0) {
-    ife->ddpaddr=ifr.ifr_addr;
-    ife->has_ddp=1;
-  }
-#endif
-
-#if HAVE_AFIPX  
-  /* Look for IPX addresses with all framing types */
-  strcpy(ifr.ifr_name, ifname);
-  if (ipx_sock >= 0) {
-    if (!ipx_getaddr(ipx_sock, IPX_FRAME_ETHERII, &ifr)) {
-      ife->has_ipx_bb=1;
-      ife->ipxaddr_bb=ifr.ifr_addr;
-    }
-    strcpy(ifr.ifr_name, ifname);
-    if (!ipx_getaddr(ipx_sock, IPX_FRAME_SNAP, &ifr)) {
-      ife->has_ipx_sn=1;
-      ife->ipxaddr_sn=ifr.ifr_addr;
-    }
-    strcpy(ifr.ifr_name, ifname);
-    if(!ipx_getaddr(ipx_sock, IPX_FRAME_8023, &ifr)) {
-      ife->has_ipx_e3=1;
-      ife->ipxaddr_e3=ifr.ifr_addr;
-    }
-    strcpy(ifr.ifr_name, ifname);
-    if(!ipx_getaddr(ipx_sock, IPX_FRAME_8022, &ifr)) {
-      ife->has_ipx_e2=1;
-      ife->ipxaddr_e2=ifr.ifr_addr;
-    }
-  }
-#endif
-
-  if_getstats(ifname,ife);
-  return(0);
-}
-
-
 static void
 if_print(char *ifname)
 {
   struct interface ife;
-  struct ifconf ifc;
-  struct ifreq *ifr;
-  int i;
 
   if (ifname == (char *)NULL) {
-    if (ioctl(skfd, SIOCGIFCOUNT, &i) < 0) {
-      int n = 2, s;
-      ifc.ifc_buf = NULL;
-      do {
-	n *= 2;
-	ifc.ifc_buf = realloc(ifc.ifc_buf, (ifc.ifc_len = s = 
-					    n*sizeof(struct ifreq)));
-	if (ifc.ifc_buf == NULL) {
-	  fprintf(stderr, "Out of memory\n");
-	  exit(1);
-	}
-	
-	if (ioctl(skfd, SIOCGIFCONF, &ifc) < 0) {
-	  perror("SIOCGIFCONF");
-	  return;
-	}
-      } while (ifc.ifc_len == s);
-    } else {
-      ifc.ifc_buf = malloc(ifc.ifc_len = i*sizeof(struct ifreq));
-      if (ifc.ifc_buf == NULL) {
-	fprintf(stderr, "Out of memory.\n");
-	exit(1);
-      }
-      if (ioctl(skfd, SIOCGIFCONF, &ifc) < 0) {
-	perror("SIOCGIFCONF");
-	return;
-      }
-    }
-    ifr = ifc.ifc_req;
-    for (i = ifc.ifc_len / sizeof(struct ifreq); --i >= 0; ifr++) {
-      
-      if (if_fetch(ifr->ifr_name, &ife) < 0) {
+    FILE *fd = fopen(_PATH_PROCNET_DEV, "r");
+    char buffer[256];
+    fgets(buffer, 256, fd);	/* chuck first two lines */
+    fgets(buffer, 256, fd);
+    while (!feof(fd)) {
+      char *name = buffer;
+      if (fgets(buffer, 256, fd) == NULL)
+	break;
+      buffer[6] = 0;
+      while (*name == ' ') name++;
+      if (if_fetch(name, &ife) < 0) {
 	fprintf(stderr, NLS_CATGETS(catfd, ifconfigSet, 
 		      ifconfig_unkn, "%s: unknown interface.\n"),
-		ifr->ifr_name);
+		name);
 	continue;
       }
       
       if (((ife.flags & IFF_UP) == 0) && !opt_a) continue;
       ife_print(&ife);
     }
+    fclose(fd);
   } else {
     if (if_fetch(ifname, &ife) < 0)
       fprintf(stderr, NLS_CATGETS(catfd, ifconfigSet, 
@@ -621,8 +335,6 @@ if_print(char *ifname)
     else 
       ife_print(&ife);
   }
-  
-  free(ifc.ifc_buf);
 }
 
 
@@ -633,11 +345,15 @@ set_flag(char *ifname, short flag)
   struct ifreq ifr;
 
   strcpy(ifr.ifr_name, ifname);
-  if (ioctl(skfd, SIOCGIFFLAGS, &ifr) < 0) return(-1);
+  if (ioctl(skfd, SIOCGIFFLAGS, &ifr) < 0) {
+    fprintf(stderr, "%s: unknown interface.\n", ifname);
+    return(-1);
+  }
+  strcpy(ifr.ifr_name, ifname);
   ifr.ifr_flags |= flag;
   if (ioctl(skfd, SIOCSIFFLAGS, &ifr) < 0) {
-	fprintf(stderr, "SIOCSIFFLAGS: %s\n", strerror(errno));
-	return(-1);
+    perror("SIOCSIFFLAGS");
+    return -1;
   }
   return(0);
 }
@@ -650,11 +366,15 @@ clr_flag(char *ifname, short flag)
   struct ifreq ifr;
 
   strcpy(ifr.ifr_name, ifname);
-  if (ioctl(skfd, SIOCGIFFLAGS, &ifr) < 0) return(-1);
+  if (ioctl(skfd, SIOCGIFFLAGS, &ifr) < 0) {
+    fprintf(stderr, "%s: unknown interface.\n", ifname);
+    return -1;
+  }
+  strcpy(ifr.ifr_name, ifname);
   ifr.ifr_flags &= ~flag;
   if (ioctl(skfd, SIOCSIFFLAGS, &ifr) < 0) {
-	fprintf(stderr, "SIOCSIFFLAGS: %s\n", strerror(errno));
-	return(-1);
+    perror("SIOCSIFFLAGS");
+    return -1;
   }
   return(0);
 }
@@ -668,7 +388,9 @@ usage(void)
   fprintf(stderr, "                [[family] address]\n");
 #if HAVE_AFINET6
   fprintf(stderr, "                [add inet6address/prefixlen]\n");
+#ifdef SIOCDIFADDR
   fprintf(stderr, "                [del inet6address/prefixlen]\n");
+#endif
   fprintf(stderr, "                [tunnel aa.bb.cc.dd]\n");
 #endif
 #if HAVE_AFINET
@@ -684,6 +406,9 @@ usage(void)
   fprintf(stderr, "                [multicast]\n");
   fprintf(stderr, "                [mem_start NN] [io_addr NN] [irq NN]\n");
   fprintf(stderr, "                [media type]\n");
+#ifdef HAVE_TXQUEUELEN
+  fprintf(stderr, "                [txqueuelen len]\n");
+#endif
   fprintf(stderr, "                [up] [down] ...\n");
   NLS_CATCLOSE(catfd)
   exit(1);
@@ -697,57 +422,6 @@ version(void)
   exit(1);
 }
 
-static int sockets_open()
-{
-#if HAVE_AFINET
-  inet_sock = socket(AF_INET, SOCK_DGRAM, 0);
-#endif
-
-#if HAVE_AFINET6
-  inet6_sock = socket(AF_INET6, SOCK_DGRAM, 0);
-#endif
-
-#if HAVE_AFIPX 
-  ipx_sock = socket(AF_IPX, SOCK_DGRAM, 0);
-#endif
-
-#if HAVE_AFAX25 
-  ax25_sock = socket(AF_AX25, SOCK_DGRAM, 0);
-#endif
-
-#if HAVE_AFATALK
-  ddp_sock = socket(AF_APPLETALK, SOCK_DGRAM, 0);
-#endif
-  
-  /*
-   *	Now pick any (existing) useful socket family for generic queries
-   */
-
-#if HAVE_AFINET
-  if (inet_sock != -1) return inet_sock;
-#endif
-
-#if HAVE_AFINET6
-  if (inet6_sock != -1) return inet6_sock;
-#endif
-
-#if HAVE_AFIPX 
-  if (ipx_sock != -1) return ipx_sock;
-#endif
-
-#if HAVE_AFAX25 
-  if (ax25_sock != -1) return ax25_sock;
-#endif
-
-#if HAVE_AFATALK
-  if (ddp_sock != -1) return ddp_sock;
-#endif
-
-  /* We have no address families.  */
-  fprintf(stderr, "No usable address families found.\n");
-  return -1;
-}
-	
 int
 main(int argc, char **argv)
 {
@@ -864,7 +538,7 @@ main(int argc, char **argv)
 	  }
 	  ifr.ifr_map.port = newport;
 	  if (ioctl(skfd, SIOCSIFMAP, &ifr) < 0) {
-	    fprintf(stderr, "SIOCSIFMAP: %s\n", strerror(errno));
+	    perror("SIOCSIFMAP");
 	    goterr = 1;
 	  }
 	}
@@ -925,7 +599,7 @@ main(int argc, char **argv)
       spp++;
       continue;
     }
-    
+
     if (!strcmp(*spp, "down")) {
       goterr |= clr_flag(ifr.ifr_name, IFF_UP);
       spp++;
@@ -1021,7 +695,20 @@ main(int argc, char **argv)
       spp++;
       continue;
     }
-    
+
+#ifdef HAVE_TXQUEUELEN
+    if (!strcmp(*spp, "txqueuelen")) {
+      if (*++spp == NULL) usage();
+      ifr.ifr_qlen = strtoul(*spp, NULL, 0);
+      if (ioctl(skfd, SIOCSIFTXQLEN, &ifr) < 0) {
+	fprintf(stderr, "SIOCSIFTXQLEN: %s\n", strerror(errno));
+	goterr = 1;
+      }
+      spp++;
+      continue;
+    }
+#endif
+
     if (!strcmp(*spp, "mem_start")) {
       if (*++spp == NULL) usage();
       if (ioctl(skfd, SIOCGIFMAP, &ifr) < 0) {
@@ -1184,11 +871,15 @@ main(int argc, char **argv)
 
       ifr6.ifr6_ifindex = ifr.ifr_ifindex;
       ifr6.ifr6_prefixlen = prefix_len;
+#ifdef SIOCDIFADDR
       if (ioctl(inet6_sock, SIOCDIFADDR, &ifr6) < 0) {
 	fprintf(stderr, "SIOCDIFADDR: %s\n",
 		strerror(errno));
 	goterr = 1;
       }
+#else
+      fprintf(stderr, "Address deletion not supported on this system.\n");
+#endif
       spp++;
       continue;
     }
